"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificationRequest = void 0;
const Moment = __importStar(require("moment"));
const moment_range_1 = require("moment-range");
const utils_1 = require("ethers/utils");
const polly_js_1 = __importDefault(require("polly-js"));
const origin_backend_core_1 = require("@energyweb/origin-backend-core");
const PreciseProofEntity_1 = require("./PreciseProofEntity");
const contracts_1 = require("../contracts");
class CertificationRequest extends PreciseProofEntity_1.PreciseProofEntity {
    constructor(id, configuration, isPrivate = false) {
        super(id, configuration);
        this.isPrivate = isPrivate;
        this.initialized = false;
    }
    static async create(fromTime, toTime, energy, deviceId, configuration, files, isVolumePrivate = false, forAddress) {
        if (energy.gt(origin_backend_core_1.MAX_ENERGY_PER_CERTIFICATE)) {
            throw new Error(`Too much energy requested. Requested: ${energy}, Max: ${origin_backend_core_1.MAX_ENERGY_PER_CERTIFICATE}`);
        }
        const request = new CertificationRequest(null, configuration, isVolumePrivate);
        const { issuer } = configuration.blockchainProperties;
        const issuerWithSigner = issuer.connect(configuration.blockchainProperties.activeUser);
        await this.validateGenerationPeriod(fromTime, toTime, deviceId, configuration);
        const success = await configuration.offChainDataSource.certificateClient.queueCertificationRequestData({ deviceId, fromTime, toTime, energy, files });
        if (!success) {
            throw new Error('Unable to create CertificationRequest');
        }
        const data = await issuer.encodeData(fromTime, toTime, deviceId);
        const tx = await (forAddress
            ? issuerWithSigner.requestCertificationFor(data, forAddress, isVolumePrivate)
            : issuerWithSigner.requestCertification(data, isVolumePrivate));
        const { events: [certificationRequested] } = await tx.wait();
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        request.id = certificationRequested.args._id.toNumber();
        if (configuration.logger) {
            configuration.logger.info(`CertificationRequest ${request.id} created`);
        }
        return request.sync();
    }
    async sync() {
        const offChainData = await polly_js_1.default()
            .waitAndRetry([2000, 4000, 8000, 16000])
            .executeForPromise(() => this.configuration.offChainDataSource.certificateClient.getCertificationRequest(this.id));
        Object.assign(this, offChainData);
        this.initialized = true;
        if (this.configuration.logger) {
            this.configuration.logger.verbose(`CertificationRequest ${this.id} synced`);
        }
        return this;
    }
    async approve() {
        const issuerInterface = new utils_1.Interface(contracts_1.IssuerJSON.abi);
        const validityData = issuerInterface.functions.isRequestValid.encode([this.id.toString()]);
        let approveTx;
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        if (this.isPrivate) {
            const commitment = {
                [this.owner]: this.energy
            };
            const { rootHash } = this.generateAndAddProofs(commitment);
            approveTx = await issuerWithSigner.approveCertificationRequestPrivate(this.id, rootHash, validityData);
        }
        else {
            approveTx = await issuerWithSigner.approveCertificationRequest(this.id, this.energy, validityData);
        }
        const { events } = await approveTx.wait();
        const certificateId = Number(events.find((log) => log.event === 'CertificationRequestApproved')
            .topics[3]);
        return certificateId;
    }
    async revoke() {
        const { issuer } = this.configuration
            .blockchainProperties;
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const revokeTx = await issuerWithSigner.revokeRequest(this.id);
        await revokeTx.wait();
    }
    async requestMigrateToPublic(value) {
        if (!this.isPrivate) {
            throw new Error('Certificate is already public.');
        }
        const { issuer } = this.configuration
            .blockchainProperties;
        const activeUserAddress = await this.configuration.blockchainProperties.activeUser.getAddress();
        const commitment = {
            [activeUserAddress]: value
        };
        const { rootHash } = this.generateAndAddProofs(commitment);
        const issuerWithSigner = issuer.connect(this.configuration.blockchainProperties.activeUser);
        const tx = await issuerWithSigner.requestMigrateToPublic(this.id, rootHash);
        await tx.wait();
        return tx;
    }
    static async getAll(configuration) {
        const all = await configuration.offChainDataSource.certificateClient.getAllCertificationRequests();
        const certificationRequestPromises = all.map((certReq) => new CertificationRequest(certReq.id, configuration).sync());
        return Promise.all(certificationRequestPromises);
    }
    static async validateGenerationPeriod(fromTime, toTime, deviceId, configuration) {
        const moment = moment_range_1.extendMoment(Moment);
        const unix = (timestamp) => moment.unix(timestamp);
        const allCertificationRequests = await this.getAll(configuration);
        const generationTimeRange = moment.range(unix(fromTime), unix(toTime));
        for (const certificationRequest of allCertificationRequests) {
            if (certificationRequest.revoked || certificationRequest.deviceId !== deviceId) {
                continue;
            }
            const certificationRequestGenerationRange = moment.range(unix(certificationRequest.fromTime), unix(certificationRequest.toTime));
            if (generationTimeRange.overlaps(certificationRequestGenerationRange)) {
                throw new Error(`Generation period ` +
                    `${unix(fromTime).format()} - ${unix(toTime).format()}` +
                    ` overlaps with the time period of ` +
                    `${unix(certificationRequest.fromTime).format()} - ${unix(certificationRequest.toTime).format()}` +
                    ` of request ${certificationRequest.id}.`);
            }
        }
        return true;
    }
}
exports.CertificationRequest = CertificationRequest;
//# sourceMappingURL=CertificationRequest.js.map